pub struct Consts { /* private fields */ }Expand description
Constants cache contains arbitrary-precision mathematical constants.
Implementations
Sourceimpl Consts
In an ideal situation, the Consts structure is initialized with Consts::new only once,
and then used where needed.
impl Consts
In an ideal situation, the Consts structure is initialized with Consts::new only once,
and then used where needed.
Sourcepub fn pi(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn pi(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Returns the value of the pi number with precision p using rounding mode rm.
Precision is rounded upwards to the word size.
Sourcepub fn e(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn e(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Returns the value of the Euler number with precision p using rounding mode rm.
Precision is rounded upwards to the word size.
Sourcepub fn ln_2(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn ln_2(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Returns the value of the natural logarithm of 2 with precision p using rounding mode rm.
Precision is rounded upwards to the word size.
Sourcepub fn ln_10(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn ln_10(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Returns the value of the natural logarithm of 10 with precision p using rounding mode rm.
Precision is rounded upwards to the word size.
Sourcepub fn cache_info(&self) -> ConstCacheInfo
pub fn cache_info(&self) -> ConstCacheInfo
How many bits of each series / extra constant are currently retained.
Sourcepub fn sqrt2(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn sqrt2(&mut self, p: usize, rm: RoundingMode) -> ExactNum
√2 with precision p using rounding mode rm.
Higher requests extend the cache; lower requests reuse it.
Sourcepub fn phi(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn phi(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Golden ratio φ = (1+√5)/2 with precision p using rounding mode rm.
Higher requests extend the cache; lower requests reuse it.
Sourcepub fn euler_gamma(&mut self, p: usize, rm: RoundingMode) -> ExactNum
pub fn euler_gamma(&mut self, p: usize, rm: RoundingMode) -> ExactNum
Euler–Mascheroni constant γ with precision p using rounding mode rm.